RED_COLOR = '#ed8796'
PINK_COLOR = '#ee99a0'
ORANGE_COLOR = '#f5a97f'
YELLOW_COLOR = '#eed49f'
GREEN_COLOR = '#a6da95'
AQUAMARINE_COLOR = '#8bd5ca'
LAVENDER_COLOR = '#b7bdf8'
PURPLE_COLOR = '#c6a0f6'
WHITE_COLOR = '#fff'
RESET = '\u001b[0m'